"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _config = _interopRequireDefault(require("../../config/config"));

var _utils = require("../utils");

var _validates = require("../validates");

var _common = require("./common");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const {
  INSTALL_DIR_PATH,
  DEFAULT_OUT_DIR_PATH,
  CONFIG_FILE_PATH,
  PACKAGE_JSON_FILE_PATH
} = (0, _config.default)(process.env.NODE_ENV);

const initialize = async () => {
  if (!(await _utils.dataControl.isFileExisted())) {
    await _utils.dataControl.init();
  }
};

const list = async () => {
  try {
    await initialize();
    const templates = await _utils.dataControl.read();
    Object.entries(templates).forEach(arr => {
      if (arr[1].default) {
        console.log(`* ${arr[0]}`);
      } else {
        console.log(`  ${arr[0]}`);
      }
    });
  } catch (error) {
    console.error(`ERROR: ${error}`);
  }
};

const show = async templateName => {
  try {
    await initialize();
    const templates = await _utils.dataControl.read();
    if (!Object.keys(templates).includes(templateName)) throw new Error(`Not such a templateName: '${templateName}'`);
    (0, _common.prettyLog)(templates[templateName]);
  } catch (error) {
    console.error(`ERROR: ${error}`);
  }
};

const setDefault = async templateName => {
  try {
    await initialize();
    const templates = await _utils.dataControl.read();
    if (!Object.keys(templates).includes(templateName)) throw new Error(`Not such a templateName: '${templateName}'`);
    const newTemplates = {};
    Object.entries(templates).forEach(arr => {
      const name = arr[0];
      const data = arr[1];

      if (name === templateName) {
        data.default = true;
      } else {
        data.default = false;
      }

      newTemplates[name] = data;
    });
    await _utils.dataControl.write(newTemplates);
    console.log(`The default was changed to '${templateName}'`);
  } catch (error) {
    console.error(`ERROR: ${error}`);
  }
};

const install = async templateName => {
  try {
    await initialize();
    await (0, _common.initConfigFile)();
    const templates = await _utils.dataControl.read();
    const configFile = await _utils.dataControl.read({
      path: CONFIG_FILE_PATH
    });
    let targetTemplateName = templateName;

    if (targetTemplateName) {
      if (!Object.keys(templates).includes(targetTemplateName)) throw new Error(`Not such a templateName: '${targetTemplateName}'`);
    } else {
      // default
      Object.entries(templates).forEach(t => {
        const name = t[0];
        const templateObj = t[1];

        if (templateObj.default && templateObj.default === true) {
          targetTemplateName = name;
        }
      });
      if (!targetTemplateName) throw new Error(`default is not set`);
    }

    const isYarn = configFile.environment === _common.PACKAGE_MANAGERS.YARN;

    if (await _utils.dataControl.isFileExisted({
      path: PACKAGE_JSON_FILE_PATH
    })) {
      await _utils.npmExec.init({
        isYarn
      });
    }

    const targetTemplate = templates[targetTemplateName];
    const depModuleNames = Object.entries(targetTemplate.dependencies || {}).map(dep => `${dep[0]}@${dep[1]}`);
    const depPromises = depModuleNames.map(moduleName => () => _utils.npmExec.add(moduleName, {
      isYarn
    }));
    const devDepModuleNames = Object.entries(targetTemplate.devDependencies || {}).map(dep => `${dep[0]}@${dep[1]}`);
    const devDepPromises = devDepModuleNames.map(moduleName => () => _utils.npmExec.add(moduleName, {
      isDev: true,
      isYarn
    }));
    const filePromises = Object.entries(targetTemplate.files || {}).map(file => async () => {
      try {
        const fileName = file[0];
        const fileContent = file[1];
        const fileExtension = fileName.split('.').pop();
        const path = `${INSTALL_DIR_PATH}/${fileName}`;
        await _utils.dataControl.write(fileContent, {
          path,
          fileExtension
        });
        console.log(`New file created at: '${path}'`);
        return Promise.resolve(true);
      } catch (error) {
        throw error;
      }
    });
    await (0, _common.promiseAllSerially)(depPromises);
    await (0, _common.promiseAllSerially)(devDepPromises);
    await (0, _common.promiseAllParallelly)(filePromises);
    console.log(`The dependencies, devDependencies and files were installed`);
  } catch (error) {
    console.error(`ERROR: ${error}`);
  }
};

const add = async (templateName, templateJsonString) => {
  try {
    await initialize();
    const templateObj = JSON.parse(templateJsonString);
    const templates = await _utils.dataControl.read();

    if (Object.keys(templates).length === 0) {
      templateObj.default = true;
    } else {
      templateObj.default = false;
    }

    const {
      errors
    } = (0, _validates.validateTemplate)(templateObj);
    if (errors.length > 0) throw new Error(errors.join(','));
    await _utils.dataControl.add(templateName, templateObj);
    console.log(`New template was added: '${templateName}'`);
  } catch (error) {
    console.error(`ERROR: ${error}`);
  }
};

const addFromFile = async (templateName, templateFilePath) => {
  try {
    const templateObj = await _utils.dataControl.read({
      path: templateFilePath
    });
    await add(templateName, JSON.stringify(templateObj));
  } catch (error) {
    console.error(`ERROR: ${error}`);
  }
};

const remove = async templateName => {
  try {
    await initialize();
    const templates = await _utils.dataControl.read();
    if (!Object.keys(templates).includes(templateName)) throw new Error(`Not such a templateName: '${templateName}'`);
    await _utils.dataControl.remove(templateName);
    console.log(`The template was removed: '${templateName}'`);
  } catch (error) {
    console.error(`ERROR: ${error}`);
  }
};

const rename = async (templateName, newTemplateName) => {
  try {
    await initialize();
    const templates = await _utils.dataControl.read();
    if (!Object.keys(templates).includes(templateName)) throw new Error(`Not such a templateName: '${templateName}'`);
    const newTemplateObj = templates[templateName];
    await _utils.dataControl.remove(templateName);
    await _utils.dataControl.add(newTemplateName, newTemplateObj);
    console.log(`The template '${templateName}' was renamed to: '${newTemplateName}'`);
  } catch (error) {
    console.error(`ERROR: ${error}`);
  }
};

const viewFile = async (templateName, viewStatement) => {
  try {
    await initialize();
    const templates = await _utils.dataControl.read();
    if (!Object.keys(templates).includes(templateName)) throw new Error(`Not such a templateName: '${templateName}'`);
    const template = templates[templateName];
    const result = viewStatement.split('.').reduce((obj, index) => obj[index], template);
    (0, _common.prettyLog)(result || '');
  } catch (error) {
    console.error(`ERROR: ${error}`);
  }
};

const updateFile = async (templateName, updateStatement) => {
  try {
    await initialize();
    const templates = await _utils.dataControl.read();
    if (!Object.keys(templates).includes(templateName)) throw new Error(`Not such a templateName: '${templateName}'`);
    const updateField = updateStatement.split('=')[0];
    const updateValueStr = updateStatement.slice(updateField.length + 1);
    const rootField = updateField.split('.')[0];
    if (!['dependencies', 'devDependencies', 'files'].includes(rootField)) throw new Error(`Cannot update the field: ${rootField}`);
    const template = templates[templateName];
    const currentValue = updateField.split('.').slice(0, -1).reduce((obj, index) => obj[index], template);
    const lastField = updateField.split('.').pop();
    const updateValue = typeof currentValue === 'object' ? Object.assign(currentValue, {
      [lastField]: JSON.parse(updateValueStr)
    }) : updateValueStr;
    const update = updateField.split('.').slice(0, -1).reverse().reduce((obj, prop) => {
      const result = obj;

      if (Object.keys(obj).length === 0) {
        result[prop] = updateValue;
        return result;
      }

      const key = Object.keys(obj)[0];
      result[prop] = Object.assign({}, obj);
      delete result[key];
      return result;
    }, {});
    const updatedTemplateObj = Object.assign(template, update);
    const {
      errors
    } = (0, _validates.validateTemplate)(updatedTemplateObj);
    if (errors.length > 0) throw new Error(errors.join(','));
    await _utils.dataControl.remove(templateName);
    await _utils.dataControl.add(templateName, updatedTemplateObj);
    console.log(`The template was updated: '${templateName}'`);
  } catch (error) {
    console.error(`ERROR: ${error}`);
  }
};

const exportFile = async (templateName, filename, dirPath) => {
  try {
    await initialize();
    const templates = await _utils.dataControl.read();
    if (!Object.keys(templates).includes(templateName)) throw new Error(`Not such a templateName: '${templateName}'`);
    const outFileName = filename || `${templateName}-template.json`;
    const outDirPath = dirPath || DEFAULT_OUT_DIR_PATH;
    const path = `${outDirPath}/${outFileName}`;
    const outTemplateObj = templates[templateName];
    delete outTemplateObj.default;

    if (!(await _utils.dataControl.isDirExisted({
      dirPath: outDirPath
    }))) {
      throw new Error(`Not such a directory: '${outDirPath}'`);
    }

    await _utils.dataControl.write(outTemplateObj, {
      path
    });
    console.log(`The template '${templateName}' was exported to: '${path}'`);
  } catch (error) {
    console.error(`ERROR: ${error}`);
  }
};

var _default = {
  list,
  show,
  setDefault,
  install,
  add,
  addFromFile,
  remove,
  rename,
  viewFile,
  updateFile,
  exportFile
};
exports.default = _default;